<?php

namespace Controllers;

use Controllers\ReportWorkflowController as Report;
use Services\AprovacaoAlcadaService as Service;
use Controllers\SupplyTekApprovalWorkflowController as SupplyTekApproval;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

class AprovacaoAlcadasController
{
    private $totalAFAtual = 0;
    private $valorDemanda = 0;
    private $valorAprovacaoAtual;
    private $supplyTekApproval;
    private $totalOrderValue = 0;

    public function __construct(Service $service, Report $report, SupplyTekApproval $supplyTekApproval)
    {
        $this->service = $service;
        $this->report = $report;
        $this->supplyTekApproval = $supplyTekApproval;
    }

    public function getRequestToAprrove(Row $request, Response $response)
    {
        $row = $request->getRow();
        $row["CDOPERLOG"] = $this->service->getOperador();

        try {
            if (isset($row["IDTPALCADA"])) {
                if ($row["IDTPALCADA"] === "CATALOGO") {
                    //alçada do tipo não consiste orçamento
                    $this->naoConsisteOrcamento($row);
                } else {
                    //alçada do tipo consiste orçamento
                    $this->consisteOrcamento($row, $response);
                }
            }

            if ($row['OPERACAO'] === 'APROVAR') {
                $messageToResponse = "SUCCESS_APPROVED";
            } else if ($row['OPERACAO'] === 'REPROVAR') {
                $messageToResponse = "SUCCESS_REPROVED";
            }

            $messageToResponse = new Response\Message($messageToResponse);
            $response->addMessage($messageToResponse);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function finalizeWorkflow($row, $status)
    {
        try {
            $this->service->updateOrder($row, $status);
            if(isset($row['IDTPALCADA']) && $row['IDTPALCADA'] === 'ORCAMENTO') {
                $this->service->executeCONSISTE_ORCAMENTO($row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN']);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function naoConsisteOrcamento($row)
    {
        try {
            if ($row['IDTPDEMANDA'] === 'AF') {
                if ($row['OPERACAO'] == 'APROVAR') {
                    $nextAprovador = $this->service->sqlApprover($row);
                    
                    if (isset($nextAprovador) && count($nextAprovador)) {
                        //não consiste orçamento
                        
                        $this->service->updateWorkflow($row, 'APROVADO');
                        $this->finalizeWorkflow($row, 'PENDENTE');
                        
                        $row['IDCLASSIFIC'] = 'PADRAO';
                        $row['IDOPERACAOALC'] = 'LANCAMENTO';
                        $row['IDAPROVADO'] = 'PENDENTE';
                        
                        $approver = end($nextAprovador)['CDOPERADOR'];
                        $this->service->insertAprovalcadaSQL($row, $row['NRCONFIGALCADA'], $approver);

                        //TODO tratar envio de email
                        //$this->report->sendApprovalReport($orderData, 'APROVACAO_ALCADA', $justifyMessage, $row['CDOPERLOG']);
                    } else {
                        //consiste orçamento
                        $procedure = $this->service->executeCONSISTE_ORCAMENTO($row["CDFILDEST"], $row["CDFILLANC"], $row["NRAUTOFORN"]);
                        $consiste = isset($procedure['P_CONSISTE']) ? $procedure['P_CONSISTE'] : null;
                        if ($consiste === 'N') {
                            $data = $this->service->getConsisteOrcamento($row);
                            if (isset($data) && count($data)) {
                                $this->validatesApprover($row, $data);

                                $this->service->updateWorkflow($row, 'APROVADO');
                                $this->finalizeWorkflow($row, 'PENDENTE');
                                $orcamento = end($data);
                                $this->setFirstApprover($row, $orcamento);
                            } else {
                                $this->finalizeWorkflow($row, 'SEM_ALCADA');
                                throw new \Exception("NO_ALCADA_WORKFLOW");
                            }
                        } else {
                            $this->service->updateWorkflow($row, 'APROVADO');
                            $this->finalizeWorkflow($row, 'APROVADA');
                        }
                    }
                } else {
                    $this->service->updateWorkflow($row, 'RECUSADO');
                    $this->finalizeWorkflow($row, 'RECUSADA');

                    //TODO tratar envio de email
                    //$this->report->sendApprovalReport($orderData, 'REPROVACAO_ALCADA', $justifyMessage, $row['CDOPERLOG']);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getApproverMaxValue($row, $orcamento)
    {
        $params = array(
            'CDFILDEST' => $row['CDFILDEST'],
            'NRCONFIGALCADA' => $orcamento['NRCONFIGALCADA'],
        );
        $data = $this->service->getApproversMaxValue($params);

        if (isset($data) && count($data)) {
            $maxvalue = $data['VALORMAXIMO'];

            if ($row['IDTIPOVRAPROV'] === 'PERCENTUAL') {
                $maxvalue = ($orcamento["VRORCAMENTO"] * $maxvalue) / 100;
            }
        } else {
            $maxvalue = null;
        }

        return $maxvalue;
    }

    private function setApproverException($idconsistvraprov)
    {
        if ($idconsistvraprov === 'M') {
            throw new \Exception('N_VALUE_APPROVED_TOTAL_AF_PERIOD');
        }

        throw new \Exception('N_VALUE_APPROVED_TOTAL_AF');
    }

    private function setFirstApprover($row, $dataOrca)
    {
        try {
            $row['IDCLASSIFIC'] = 'PADRAO';
            $row['IDOPERACAOALC'] = 'LANCAMENTO';
            $row['IDAPROVADO'] = 'PENDENTE';

            $this->service->deleteAprovalcada($row, $dataOrca);
            $approver  = $this->service->getFirstApprover($row, $dataOrca);
            $this->service->insertAprovalcadaSQL($row, $dataOrca['NRCONFIGALCADA'], $approver);

            //TODO tratar envio de email 
            //$this->report->sendNotification($row, 'APROVACAO_ALCADA', null, $row['CDOPERLOG']);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function consisteOrcamento($row, $response)
    {
        try {
            if ($row['OPERACAO'] === 'APROVAR') {
                $this->orderApprove($row);
            } else if ($row['OPERACAO'] === 'REPROVAR') {
                $this->orderRepprove($row);
            }

            $this->updateOrca($row);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function validatesApprover($row, $data)
    {
        try {
            $orcamento = end($data);
            $maxValue = $this->getApproverMaxValue($row, $orcamento);
            if (!$maxValue) {
                $this->setApproverException($row["IDCONSISTVRAPROV"]);
            }
            $usaPorcentagem = $row['IDTIPOVRAPROV'] === 'PERCENTUAL';
            $this->totalOrderValue = floatval($this->getTotal($row, $row["IDCONSISTVRAPROV"], $row, $usaPorcentagem));
            if ($this->totalOrderValue > $maxValue) {
                $this->setApproverException($row["IDCONSISTVRAPROV"]);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function orderApprove($row)
    {
        try {
            $procedure = $this->service->executeCONSISTE_ORCAMENTO($row["CDFILDEST"], $row["CDFILLANC"], $row["NRAUTOFORN"]);
            $consiste = isset($procedure['P_CONSISTE']) ? $procedure['P_CONSISTE'] : null;
            if ($consiste === 'N') {
                $data = $this->service->getConsisteOrcamento($row);
                if (isset($data) && count($data)) {
                    $this->validatesApprover($row, $data);

                    $this->service->updateWorkflow($row, 'APROVADO');
                    
                    $data = $this->service->getNextApprover($row, $this->totalOrderValue);
                    $approver = isset($data) && count($data) ? end($data)['CDOPERADOR'] : null;
                                        
                    if (isset($approver)) {
                        $row['IDCLASSIFIC'] = 'PADRAO';
                        $row['IDOPERACAOALC'] = 'LANCAMENTO';
                        $row['IDAPROVADO'] = 'PENDENTE';
                        
                        $this->service->insertAprovalcadaSQL($row, $row['NRCONFIGALCADA'], $approver);
                        
                        $this->finalizeWorkflow($row, 'PENDENTE');
                        //TODO tratar envio de email 
                        //$this->report->sendNotification($row, 'APROVACAO_ALCADA', null, $row['CDOPERLOG']);
                    } else {
                        $this->finalizeWorkflow($row, 'APROVADA');

                        //TODO Tratar envio de email de liberação de demanda
                        //'LIBERACAO_DEMANDA';
                    }

                } else {
                    $this->finalizeWorkflow($row, 'SEM_ALCADA');
                    throw new \Exception("NO_ALCADA_WORKFLOW");
                }
            } else {
                $this->service->updateWorkflow($row, 'APROVADO');
                $this->finalizeWorkflow($row, 'APROVADA');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function orderRepprove($row)
    {
        try {
            $this->service->updateWorkflow($row, 'RECUSADO');
            $this->finalizeWorkflow($row, 'RECUSADA');

            //TODO Tratar envio de email
            //$this->report->sendRepproveReport($orderData, $subjectParam, $justifyMessage, $approvalData['CDOPERLOG']);            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getDataEntregaAF($dadosDemanda)
    {
        $params = [
            'CDFILDEST' => $dadosDemanda['CDFILDEST'],
            'CDFILLANC' => $dadosDemanda['CDFILLANC'],
            'NRDEMANDA' => $dadosDemanda['NRAUTOFORN'],
        ];

        return $this->service->findDataEntregaAF($params)[0]['DTPROGENTAF'];
    }

    private function getDiaInicioOrcamento($dadosDemanda)
    {
        return $this->service->findDiaInicioOrcamento($dadosDemanda['CDFILDEST'])[0]['DIAINIORCA'];
    }

    private function getDataOrcamento($diaInicioOrcamento, $dataEntregaAF)
    {
        $params = [
            'DIA' => $diaInicioOrcamento,
            'DATA' => $dataEntregaAF,
        ];

        return $this->service->findDataOrcamento($params)[0];
    }

    private function getPeriodoOrcamento($dadosDemanda, $dataEntregaAF)
    {
        $diaInicioOrcamento = $this->getDiaInicioOrcamento($dadosDemanda);

        $dataOrcamento = $this->getDataOrcamento($diaInicioOrcamento, $dataEntregaAF);

        $paramsPeriodoOrcamento = [
            'V_DATAINI' => $dataOrcamento['V_DATAINI'],
            'DIAINIORCA' => $diaInicioOrcamento,
        ];

        return $this->service->findPeriodoOrcamento($paramsPeriodoOrcamento)[0];
    }

    private function getValorDemanda($dadosDemanda, $dataEntregaAF, $periodoOrcamento)
    {
        $paramsValorDemanda = [
            'DTPROGENTAF' => $dataEntregaAF,
            'CDFILDEST'   => $dadosDemanda['CDFILDEST'],
            'NRDEMANDA'   => $dadosDemanda['NRAUTOFORN'],
            'IDPERIODOORCA'   => $periodoOrcamento['V_IDPERIODOORCA'],
            'NRPERIODOORCA'   => $periodoOrcamento['V_NRPERIODOORCAGOP'],
        ];

        return $this->service->getValorDemanda($paramsValorDemanda)[0]['VRDEMANDA'];
    }

    private function getTotalGastoNoPeriodo()
    {
        return $this->valorDemanda + $this->totalAFAtual;
    }

    private function createDATACASE($diainiorca, $dtprogentaf)
    {
        $mes = substr($dtprogentaf, 3, 2);
        $ano = substr($dtprogentaf, 6, 4);
        $ultimo_dia = date("t", mktime(0, 0, 0, $mes, '01', $ano));
        $resultado = $diainiorca > $ultimo_dia ? $ultimo_dia : $resultado = substr($diainiorca, 0, 2);
        $resultado = strlen($resultado) == 1 ? $resultado = '0' . $resultado : $resultado;

        return $resultado;
    }

    private function buscaOrcamento($dadosDemanda)
    {
        $dadosOrcamento['CDFILDEST']  = $dadosDemanda['CDFILDEST'];
        $dadosOrcamento['CDFILLANC']  = $dadosDemanda['CDFILLANC'];
        $dadosOrcamento['NRAUTOFORN'] = $dadosDemanda['NRAUTOFORN'];

        $diainiorca = $this->service->buscaDIAINIORCA($dadosOrcamento);
        $dadosOrcamento['DIAINIORCA'] = $diainiorca[0]['DIAINIORCA'];

        $dtprogentaf = $this->service->buscaDTPROGENTAF($dadosOrcamento);
        $dadosOrcamento['DTPROGENTAF'] = substr($dtprogentaf[0]['DTPROGENTAF'], 0, 10);

        $datacase = $this->createDATACASE($dadosOrcamento['DIAINIORCA'], $dadosOrcamento['DTPROGENTAF']);
        $dadosOrcamento['DATACASE'] = $datacase;


        $dataini = $this->service->buscaDATAINI($dadosOrcamento);
        $dadosOrcamento['DATAINI'] = substr($dataini[0]['DATAINI'], 0, 10);


        $datafim = $this->service->createDATAFIM($dadosOrcamento);
        $dadosOrcamento['DATAFIM'] = substr($datafim[0]['DATAFIM'], 0, 10);


        $periodo = $this->service->buscaPeriodo($dadosOrcamento);
        $dadosOrcamento['NRPERIODO'] = $periodo[0]['MAX_NRPERIODO'];
        $dadosOrcamento['IDPERIODOORCA'] = $periodo[0]['MAX_IDPERIODOORCA'];


        $orcamento = $this->service->buscaOrcamento($dadosOrcamento);

        if ($orcamento[0]['VRORCAITEM'] == 0) {
            throw new \Exception('N_VALUE_AF_SELECT');
        }

        return $orcamento[0]['VRORCAITEM'];
    }

    private function buscaValorTotalPorcentagem($dadosDemanda, $porcentagemAprovador)
    {
        $orcamento = $this->buscaOrcamento($dadosDemanda);
        $vraprovcmp = $orcamento * $porcentagemAprovador / 100;
        return $vraprovcmp;
    }

    private function getTotal($dadosDemanda, $paramValorUtilizado, $approvalData, $usaPorcentagem = false)
    {
        $aprovadorAtual = $this->service->findAprovadorAtual($approvalData)[0];

        if ($usaPorcentagem) {
            $aprovadorAtual['VRAPROVCMP'] = $this->buscaValorTotalPorcentagem($dadosDemanda, $aprovadorAtual['VRAPROVCMP']);
        }

        $this->totalAFAtual = $dadosDemanda['VRTOTALAF'];

        if ($paramValorUtilizado == 'M') {
            $this->valorAprovacaoAtual = $aprovadorAtual['VRAPROVCMP'];
            $dataEntregaAF    = $this->getDataEntregaAF($dadosDemanda);
            $periodoOrcamento = $this->getPeriodoOrcamento($dadosDemanda, $dataEntregaAF);
            $this->valorDemanda = $this->getValorDemanda($dadosDemanda, $dataEntregaAF, $periodoOrcamento);
            $totalGasto = $this->getTotalGastoNoPeriodo();
        } else {
            $this->valorDemanda = 0;
            $this->valorAprovacaoAtual = $aprovadorAtual['VRAPROVCMP'];
            $totalGasto = $dadosDemanda['VRTOTALAF'];
        }
        return $totalGasto;
    }

    private function updateOrca($row)
    {
        $autoforne = $this->service->getOrder($row);

        $params = [
            'CDFILDEST' => $autoforne['CDFILDEST'],
            'CDFILLANC' => $autoforne['CDFILLANC'],
            'NRDEMANDA' => $autoforne['NRAUTOFORN'],
        ];

        $dtEntregas = $this->service->findDataEntregaAF($params);

        foreach ($dtEntregas as $dtEntr) {
            //Execução para atualizar o orçamento com a demanda atual aprovada
            $this->service->executeCONSISTE_ORCAMENTO($autoforne['CDFILDEST'], $autoforne['CDFILLANC'], null, $dtEntr['DTPROGENTAF']);
        }
    }
}
