<?php
namespace Modules\Cadastros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Util\SupUtilities;

class CadastroUsuarioSupplyTekService {
    public function __construct(SupUtilities $supUtilities) {
        $this->supUtilities = $supUtilities;
    }

    public function insertUserSupplyTek($row) {
        try {
            $query = "INSERT INTO ACESSOSUPPLY
                        (CDFILIAL, CDOPERADOR, IDPERFILACES, IDATIVO)
                      VALUES
                        (:CDFILIAL, :CDOPERADOR, :IDPERFILACES, :IDATIVO)";

            foreach($row['CDFILIAL'] as $filial) {

                $params = [
                    'CDFILIAL'          => $filial['CDFILIAL'],
                    'CDOPERADOR'        => $row['CDOPERADOR'],
                    'IDPERFILACES'      => $row['IDPERFILACES'],
                    'IDATIVO'           => $row['IDATIVO']
                ];

                $this->supUtilities->crud($query, $params);
            }
        }
        catch (Exception $e) {
             throw $e;
        }
    }

    public function saveUnidadePadrao($row) {
        try {
            $query = "UPDATE ACESSOSUPPLY
                        SET IDFILIALPADRAO = :P_IDFILIALPADRAO
                    WHERE CDOPERADOR = :P_CDOPERADOR AND CDFILIAL = :P_CDFILIAL";

                $params = [
                    'P_CDFILIAL'          => $row['CDFILIALPADRAO'],
                    'P_CDOPERADOR'        => $row['CDOPERADOR'],
                    'P_IDFILIALPADRAO'    => 'S'
                ];
                $this->supUtilities->crud($query, $params);

        }
        catch (Exception $e) {
             throw $e;
        }
    }

    public function updateFilialNaoPadrao($row) {
        try {
            $query = "UPDATE ACESSOSUPPLY
                        SET IDFILIALPADRAO = :P_IDFILIALPADRAO
                    WHERE CDOPERADOR = :P_CDOPERADOR";

                $params = [
                    'P_CDOPERADOR'        => $row['CDOPERADOR'],
                    'P_IDFILIALPADRAO'    => 'N'
                ];
                $this->supUtilities->crud($query, $params);

        }
        catch (Exception $e) {
             throw $e;
        }
    }

    public function deleteUserSupplyTek($row) {
        try {
            $query = "DELETE FROM ACESSOSUPPLY
                       WHERE CDOPERADOR      = :P_CDOPERADOR
                         AND CDFILIAL        = :P_CDFILIAL
                         AND IDPERFILACES = :P_IDPERFILACES";

                $params = [
                    'P_CDFILIAL'        => $row['CDFILIAL'],
                    'P_CDOPERADOR'      => $row['CDOPERADOR'],
                    'P_IDPERFILACES'    => $row['IDPERFILACES']
                ];
                $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw $e;
        }
    }
    public function selectOperator($row) {
        try {

            $query = "SELECT * FROM ACESSOSUPPLY
                        WHERE CDOPERADOR = :P_CDOPERADOR";

                $params = [
                    'P_CDOPERADOR' => $row['CDOPERADOR']
                ];
                return $this->supUtilities->crud($query, $params);

        }
        catch (Exception $e) {
            throw $e;
        }
    }

    public function verifyOperator($cdOperador)
    {
        try {
            $sql = "SELECT DISTINCT
                        IDPERFILACES
                    FROM
                        ACESSOSUPPLY
                    WHERE
                        CDOPERADOR = :P_CDOPERADOR";

            $params = [
                'P_CDOPERADOR' => $cdOperador
            ];

            $result = $this->supUtilities->crud($sql, $params);

            return $result;
        }
        catch (Exception $e) {
            throw $e;
        }
    }

    public function verifyUnitFromOperator($cdOperador, $cdFilial)
    {
        try {
            $sql = "SELECT DISTINCT
                        IDPERFILACES
                    FROM
                        ACESSOSUPPLY
                    WHERE
                        CDOPERADOR = :P_CDOPERADOR
                        AND CDFILIAL = :P_CDFILIAL";

            $params = [
                'P_CDOPERADOR' => $cdOperador,
                'P_CDFILIAL' => $cdFilial
            ];

            $result = $this->supUtilities->crud($sql, $params);

            return $result;
        }
        catch (Exception $e) {
            throw $e;
        }
    }

    public function addOperator($row)
    {
        try {
            $query = "  INSERT INTO ACESSOSUPPLY
                        (
                            CDFILIAL,
                            CDOPERADOR,
                            IDPERFILACES,
                            IDATIVO,
                            IDFILIALPADRAO
                        )
                        VALUES
                        (
                            :CDFILIAL,
                            :CDOPERADOR,
                            :IDPERFILACES,
                            :IDATIVO,
                            :IDFILIALPADRAO
                        )";

            $params = [
                'CDFILIAL' => $row['CDFILIAL'],
                'CDOPERADOR' => $row['CDOPERADOR'],
                'IDPERFILACES' => $row['IDPERFILACES'],
                'IDATIVO' => $row['IDATIVO'],
                'IDFILIALPADRAO' => $row['IDFILIALPADRAO']
            ];

            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
             throw $e;
        }
    }
}

